from llama_deploy import (
    deploy_core,
    ControlPlaneConfig,
    SimpleMessageQueueConfig,
)


async def main():
    await deploy_core(
        control_plane_config=ControlPlaneConfig(),
        message_queue_config=SimpleMessageQueueConfig(),
    )


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())


import os
import tqdm
from qdrant_client import QdrantClient, models
from fastembed.embedding import TextEmbedding
from fastembed.sparse.sparse_text_embedding import SparseTextEmbedding
from fastembed.late_interaction import LateInteractionTextEmbedding
from dotenv import load_dotenv, find_dotenv
from datasets import load_dataset

_ = load_dotenv(find_dotenv())


class AdvancedHybridSearch:
    def __init__(self, collection_name: str):
        self.dense_embedding_model = TextEmbedding(model_name=os.environ.get("DENSE_MODEL"))
        self.sparse_embedding_model = SparseTextEmbedding(model_name=os.environ.get("SPARSE_MODEL"))
        self.late_interaction_embedding_model = LateInteractionTextEmbedding(os.environ.get("LATE_INTERACTION_MODEL"))

        self.client = QdrantClient(url=os.environ['DB_URL'], api_key=os.environ['DB_API_KEY'])

        self.collection_name = collection_name
        self.dense_embeddings = None
        self.sparse_embeddings = None
        self.late_interaction_embeddings = None
        self.dataset = None

        self._create_collection()

    def _get_dimensions(self):
        self.dataset = load_dataset("BeIR/scifact", 'corpus', split="corpus")
        self.dense_embeddings = list(self.dense_embedding_model.passage_embed(self.dataset["text"][0:1]))
        self.sparse_embeddings = list(self.sparse_embedding_model.passage_embed(self.dataset["text"][0:1]))
        self.late_interaction_embeddings = list(
            self.late_interaction_embedding_model.passage_embed(self.dataset["text"][0:1]))

    def _create_collection(self):

        self._get_dimensions()

        if not self.client.collection_exists(collection_name=self.collection_name):
            self.client.create_collection(
                collection_name=self.collection_name,
                vectors_config={
                    "all-MiniLM-L6-v2": models.VectorParams(
                        size=len(self.dense_embeddings[0]),
                        distance=models.Distance.COSINE
                    ),
                    "colbertv2.0": models.VectorParams(
                        size=len(self.late_interaction_embeddings[0][0]),
                        distance=models.Distance.COSINE,
                        multivector_config=models.MultiVectorConfig(
                            comparator=models.MultiVectorComparator.MAX_SIM
                        )
                    )
                },
                sparse_vectors_config={
                    "splade-PP-en-v1": models.SparseVectorParams(
                        modifier=models.Modifier.IDF
                    )
                }
            )

    def insert_data(self):
        batch_size = 4
        for batch in tqdm.tqdm(self.dataset.iter(batch_size=batch_size), total=len(self.dataset) // batch_size):
            dense_embeddings = list(self.dense_embedding_model.passage_embed(batch["text"]))
            sparse_embeddings = list(self.sparse_embedding_model.passage_embed(batch["text"]))
            late_interaction_embeddings = list(self.late_interaction_embedding_model.passage_embed(batch["text"]))

            self.client.upsert(
                collection_name=self.collection_name,
                points=[
                    models.PointStruct(
                        id=int(batch["_id"][i]),
                        vector={
                            "all-MiniLM-L6-v2": dense_embeddings[i].tolist(),
                            "splade-PP-en-v1": sparse_embeddings[i].as_object(),
                            "colbertv2.0": late_interaction_embeddings[i].tolist(),
                        },
                        payload={
                            "_id": batch["_id"][i],
                            "title": batch["title"][i],
                            "text": batch["text"][i],
                        }
                    )
                    for i, _ in enumerate(batch["_id"])
                ]
            )

    def query_with_dense_embedding(self, query_text: str):
        query_vector = next(self.dense_embedding_model.embed(query_text)).tolist()
        results = self.client.query_points(
            collection_name=self.collection_name,
            query=query_vector,
            using="all-MiniLM-L6-v2",
            with_payload=False,
            limit=10,
        )
        return results

    def query_with_sparse_embedding(self, query_text: str):
        query_vector = next(self.sparse_embedding_model.embed(query_text))
        results = self.client.query_points(
            collection_name=self.collection_name,
            query=models.SparseVector(**query_vector.as_object()),
            using="splade-PP-en-v1",
            with_payload=False,
            limit=10,
        )
        return results

    def query_with_late_interaction_embedding(self, query_text: str):
        query_vector = next(self.late_interaction_embedding_model.embed(query_text)).tolist()
        results = self.client.query_points(
            collection_name=self.collection_name,
            query=query_vector,
            using="colbertv2.0",
            with_payload=False,
            limit=10,
        )
        return results

    def query_with_rrf(self, query_text: str):
        dense_query_vector = next(self.dense_embedding_model.embed(query_text)).tolist()
        sparse_query_vector = next(self.sparse_embedding_model.embed(query_text))

        prefetch = [
            models.Prefetch(
                query=dense_query_vector,
                using="all-MiniLM-L6-v2",
                limit=20,
            ),
            models.Prefetch(
                query=models.SparseVector(**sparse_query_vector.as_object()),
                using="splade-PP-en-v1",
                limit=20,
            ),
        ]

        results = self.client.query_points(
            collection_name=self.collection_name,
            prefetch=prefetch,
            query=models.FusionQuery(
                fusion=models.Fusion.RRF
            ),
            with_payload=False,
            limit=10,
        )
        return results


from llama_index.core.workflow import Event
from llama_index. core. base. base_query_engine import BaseQueryEngine


class QueryEngineEvent(Event):
    """Result of running retrieval"""

    base_query_engine: BaseQueryEngine




import os

from llama_deploy import (
    deploy_core,
    ControlPlaneConfig
)
from llama_deploy.message_queues.rabbitmq import RabbitMQMessageQueueConfig
from dotenv import load_dotenv, find_dotenv


async def main():

    _ = load_dotenv(find_dotenv())

    await deploy_core(
        control_plane_config=ControlPlaneConfig(),
        message_queue_config=RabbitMQMessageQueueConfig(url=os.environ.get('RABBITMQ_DEFAULT_URL'),
                                                        exchange_name=os.environ.get('RABBITMQ_EXCHANGE_NAME')),
    )


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())


from llama_index.core import (
    SimpleDirectoryReader,
    VectorStoreIndex,
    StorageContext,
    Settings
)
from llama_index.core.tools import QueryEngineTool, ToolMetadata
from llama_index.embeddings.ollama import OllamaEmbedding
from llama_index.vector_stores.qdrant import QdrantVectorStore
from llama_index.core.agent import ReActAgent
from llama_index.llms.ollama import Ollama
from llama_index.core.base.response.schema import Response, StreamingResponse, AsyncStreamingResponse, PydanticResponse
from rag_evaluator import RAGEvaluator
from dotenv import load_dotenv, find_dotenv
from typing import Union
import qdrant_client
import logging
import os

_ = load_dotenv(find_dotenv())

logging.basicConfig(level=int(os.environ['INFO']))
logger = logging.getLogger(__name__)


class ReActWithQueryEngine:
    RESPONSE_TYPE = Union[
        Response, StreamingResponse, AsyncStreamingResponse, PydanticResponse
    ]

    def __init__(self, input_dir: str, similarity_top_k: int = 3, chunk_size: int = 128, chunk_overlap: int = 100,
                 show_progress: bool = False, no_of_iterations: int = 5, required_exts: list[str] = ['.pdf', '.txt']):
        self.index_loaded = False
        self.similarity_top_k = similarity_top_k
        self.input_dir = input_dir
        self._index = None
        self._engine = None
        self.agent: ReActAgent = None
        self.query_engine_tools = []
        self.show_progress = show_progress
        self.no_of_iterations = no_of_iterations
        self.required_exts = required_exts

        # use your prefered vector embeddings model
        logger.info("initializing the OllamaEmbedding")
        embed_model = OllamaEmbedding(model_name=os.environ['OLLAMA_EMBED_MODEL'],
                                      base_url=os.environ['OLLAMA_BASE_URL'])
        # openai embeddings, embedding_model_name="text-embedding-3-large"
        # embed_model = OpenAIEmbedding(embed_batch_size=10, model=embedding_model_name)

        # use your prefered llm
        llm = Ollama(model=os.environ['OLLAMA_LLM_MODEL'], base_url=os.environ['OLLAMA_BASE_URL'], request_timeout=600)
        # llm = OpenAI(model="gpt-4o")

        logger.info("initializing the global settings")
        Settings.embed_model = embed_model
        Settings.llm = llm
        Settings.chunk_size = chunk_size
        Settings.chunk_overlap = chunk_overlap

        self.rag_evaluator = RAGEvaluator()

        # Create a local Qdrant vector store
        logger.info("initializing the vector store related objects")
        self.client: qdrant_client.QdrantClient = qdrant_client.QdrantClient(url=os.environ['DB_URL'],
                                                                             api_key=os.environ['DB_API_KEY'])
        self.vector_store = QdrantVectorStore(client=self.client, collection_name=os.environ['COLLECTION_NAME'])
        self._load_data_and_create_engine()

    def _load_data_and_create_engine(self):
        if self.client.collection_exists(collection_name=os.environ['COLLECTION_NAME']):
            try:
                self._index = VectorStoreIndex.from_vector_store(vector_store=self.vector_store)
                self.index_loaded = True
            except Exception as e:
                self.index_loaded = False

        if not self.index_loaded:
            # load data
            _docs = (SimpleDirectoryReader(input_dir=self.input_dir, required_exts=self.required_exts)
                     .load_data(show_progress=self.show_progress))

            # build and persist index
            storage_context = StorageContext.from_defaults(vector_store=self.vector_store)
            logger.info("indexing the docs in VectorStoreIndex")
            self._index = VectorStoreIndex.from_documents(documents=_docs, storage_context=storage_context,
                                                          show_progress=self.show_progress)

        self._engine = self._index.as_query_engine(similarity_top_k=self.similarity_top_k)
        self._create_query_engine_tools()

    def _create_query_engine_tools(self):
        # can have more than one as per the requirement
        self.query_engine_tools.append(
            QueryEngineTool(
                query_engine=self._engine,
                metadata=ToolMetadata(
                    name="test_tool_engine",  # change this accordingly
                    description=(
                        "Provides information about user query based on the information that you have. "
                        "Use a detailed plain text question as input to the tool."
                    ),
                ),
            )
        )
        self._create_react_agent()

    def _create_react_agent(self):
        # [Optional] Add Context
        # context = """\
        # You are a stock market sorcerer who is an expert on the companies Lyft and Uber.\
        #     You will answer questions about Uber and Lyft as in the persona of a sorcerer \
        #     and veteran stock market investor.
        # """
        self.agent = ReActAgent.from_tools(
            self.query_engine_tools,
            llm=Settings.llm,
            verbose=True,
            # context=context
            max_iterations=self.no_of_iterations
        )

    def query(self, user_query: str) -> RESPONSE_TYPE:
        try:
            response = self.agent.query(str_or_query_bundle=user_query)
            if os.environ.get('IS_EVALUATION_NEEDED') == 'true':
                self.rag_evaluator.evaluate(user_query=user_query, response_obj=response)
            return response
        except Exception as e:
            logger.error(f'Error while generating response: {e}')



import os

from search import SimpleSearch
from dotenv import find_dotenv, load_dotenv

_ = load_dotenv(find_dotenv())

simple_search = SimpleSearch(collection_name=os.environ['COLLECTION_NAME'])

# open the below insert for the first time to index the data
# simple_search.insert()

result = simple_search.search(input_text="Chicago")
print(result)


import os

from llama_index.core import (
    SimpleDirectoryReader,
    VectorStoreIndex,
    StorageContext,
    Settings,
    get_response_synthesizer)
from llama_index.core.query_engine import RetrieverQueryEngine, TransformQueryEngine
from llama_index.core.node_parser import SentenceSplitter
from llama_index.core.schema import TextNode, MetadataMode
from llama_index.vector_stores.qdrant import QdrantVectorStore
from llama_index.embeddings.ollama import OllamaEmbedding
# enable if you are using openai
# from llama_index.embeddings.openai import OpenAIEmbedding
from llama_index.llms.ollama import Ollama
# enable if you are using openai
# from llama_index.llms.openai import OpenAI
from llama_index.core.retrievers import VectorIndexRetriever
from llama_index.core.indices.query.query_transform import HyDEQueryTransform
from llama_index.core.base.response.schema import Response, StreamingResponse, AsyncStreamingResponse, PydanticResponse
from rag_evaluator import RAGEvaluator
import qdrant_client
import logging
from dotenv import load_dotenv, find_dotenv
from typing import Union

_ = load_dotenv(find_dotenv())

logging.basicConfig(level=int(os.environ['INFO']))
logger = logging.getLogger(__name__)


class BaseRAG:
    RESPONSE_TYPE = Union[
        Response, StreamingResponse, AsyncStreamingResponse, PydanticResponse
    ]

    def __init__(self, data_path: str, chunk_size: int = 512, chunk_overlap: int = 200,
                 required_exts: list[str] = ['.pdf', '.txt'],
                 show_progress: bool = False, similarity_top_k: int = 3):
        # load the local data directory and chunk the data for further processing
        self.docs = SimpleDirectoryReader(input_dir=data_path, required_exts=required_exts).load_data(
            show_progress=show_progress)
        self.text_parser = SentenceSplitter(chunk_size=chunk_size, chunk_overlap=chunk_overlap)

        # Create a local Qdrant vector store
        logger.info("initializing the vector store related objects")
        client = qdrant_client.QdrantClient(url=os.environ['DB_URL'], api_key=os.environ['DB_API_KEY'])
        self.vector_store = QdrantVectorStore(client=client, collection_name=os.environ['COLLECTION_NAME'])

        # use your prefered vector embeddings model
        logger.info("initializing the OllamaEmbedding")
        embed_model = OllamaEmbedding(model_name=os.environ['OLLAMA_EMBED_MODEL'],
                                      base_url=os.environ['OLLAMA_BASE_URL'])
        # openai embeddings, embedding_model_name="text-embedding-3-large"
        # embed_model = OpenAIEmbedding(embed_batch_size=10, model=embedding_model_name)

        # use your prefered llm
        llm = Ollama(model=os.environ['OLLAMA_LLM_MODEL'], base_url=os.environ['OLLAMA_BASE_URL'], request_timeout=600)
        # llm = OpenAI(model="gpt-4o")

        logger.info("initializing the global settings")
        Settings.embed_model = embed_model
        Settings.llm = llm
        Settings.transformations = [self.text_parser]

        self.rag_evaluator = RAGEvaluator

        self.text_chunks = []
        self.doc_ids = []
        self.nodes = []

        self.similarity_top_k = similarity_top_k
        self.hyde_query_engine: TransformQueryEngine = None

        # preprocess the data like chunking, nodes, metadata etc
        self._pre_process()

    def _pre_process(self):
        logger.info("enumerating docs")
        for doc_idx, doc in enumerate(self.docs):
            curr_text_chunks = self.text_parser.split_text(doc.text)
            self.text_chunks.extend(curr_text_chunks)
            self.doc_ids.extend([doc_idx] * len(curr_text_chunks))

        logger.info("enumerating text_chunks")
        for idx, text_chunk in enumerate(self.text_chunks):
            node = TextNode(text=text_chunk)
            src_doc = self.docs[self.doc_ids[idx]]
            node.metadata = src_doc.metadata
            self.nodes.append(node)

        logger.info("enumerating nodes")
        for node in self.nodes:
            node_embedding = Settings.embed_model.get_text_embedding(
                node.get_content(metadata_mode=MetadataMode.ALL)
            )
            node.embedding = node_embedding

        # create vector store, index documents and creates retriever
        self._create_index_and_retriever()

    def _create_index_and_retriever(self):
        logger.info("initializing the storage context")
        storage_context = StorageContext.from_defaults(vector_store=self.vector_store)
        logger.info("indexing the nodes in VectorStoreIndex")
        index = VectorStoreIndex(
            nodes=self.nodes,
            storage_context=storage_context,
            transformations=Settings.transformations,
        )

        logger.info("initializing the VectorIndexRetriever with top_k as 5")
        vector_retriever = VectorIndexRetriever(index=index, similarity_top_k=self.similarity_top_k)
        response_synthesizer = get_response_synthesizer()
        logger.info("creating the RetrieverQueryEngine instance")
        vector_query_engine = RetrieverQueryEngine(
            retriever=vector_retriever,
            response_synthesizer=response_synthesizer,
        )
        logger.info("creating the HyDEQueryTransform instance")
        hyde = HyDEQueryTransform(include_original=True)
        hyde_query_engine = TransformQueryEngine(vector_query_engine, hyde)

        self.hyde_query_engine = hyde_query_engine

    def query(self, query_string: str) -> RESPONSE_TYPE:
        try:
            response = self.hyde_query_engine.query(str_or_query_bundle=query_string)
            if os.environ.get('IS_EVALUATION_NEEDED') == 'true':
                self.rag_evaluator.evaluate(user_query=query_string, response_obj=response)
            return response
        except Exception as e:
            logger.error(f'Error while inference: {e}')



from llama_index.core import (
    SimpleDirectoryReader,
    VectorStoreIndex,
    StorageContext,
    Settings
)
from llama_index.embeddings.ollama import OllamaEmbedding
from llama_index.vector_stores.qdrant import QdrantVectorStore
from llama_index.core.agent import ReActAgent
from llama_index.llms.ollama import Ollama
from llama_index.core.base.response.schema import Response, StreamingResponse, AsyncStreamingResponse, PydanticResponse
from rag_evaluator import RAGEvaluator
from dotenv import load_dotenv, find_dotenv
from typing import Union
import qdrant_client
import logging
import os
import llama_index
import phoenix as px

_ = load_dotenv(find_dotenv())

logging.basicConfig(level=int(os.environ['INFO']))
logger = logging.getLogger(__name__)

# instrumenting observability
session = px.launch_app()
llama_index.core.set_global_handler("arize_phoenix")


class SimpleRAG:
    RESPONSE_TYPE = Union[
        Response, StreamingResponse, AsyncStreamingResponse, PydanticResponse
    ]

    def __init__(self, input_dir: str, similarity_top_k: int = 3, chunk_size: int = 128, chunk_overlap: int = 100,
                 show_progress: bool = False):
        self.index_loaded = False
        self.similarity_top_k = similarity_top_k
        self.input_dir = input_dir
        self._index = None
        self._engine = None
        self.agent: ReActAgent = None
        self.query_engine_tools = []
        self.show_progress = show_progress

        self.rag_evaluator = RAGEvaluator()

        # use your prefered vector embeddings model
        logger.info("initializing the OllamaEmbedding")
        embed_model = OllamaEmbedding(model_name=os.environ['OLLAMA_EMBED_MODEL'],
                                      base_url=os.environ['OLLAMA_BASE_URL'])
        # openai embeddings, embedding_model_name="text-embedding-3-large"
        # embed_model = OpenAIEmbedding(embed_batch_size=10, model=embedding_model_name)

        # use your prefered llm
        llm = Ollama(model=os.environ['OLLAMA_LLM_MODEL'], base_url=os.environ['OLLAMA_BASE_URL'], request_timeout=600)
        # llm = OpenAI(model="gpt-4o")

        logger.info("initializing the global settings")
        Settings.embed_model = embed_model
        Settings.llm = llm
        Settings.chunk_size = chunk_size
        Settings.chunk_overlap = chunk_overlap

        # Create a local Qdrant vector store
        logger.info("initializing the vector store related objects")
        self.client: qdrant_client.QdrantClient = qdrant_client.QdrantClient(url=os.environ['DB_URL'],
                                                                             api_key=os.environ['DB_API_KEY'])
        self.vector_store = QdrantVectorStore(client=self.client, collection_name=os.environ['COLLECTION_NAME'])

        self._create_index()

    def _create_index(self):
        if self.client.collection_exists(collection_name=os.environ['COLLECTION_NAME']):
            try:
                self._index = VectorStoreIndex.from_vector_store(vector_store=self.vector_store)
                self.index_loaded = True
            except Exception as e:
                self.index_loaded = False

        if not self.index_loaded:
            # load data
            _docs = SimpleDirectoryReader(input_dir=self.input_dir).load_data(show_progress=self.show_progress)

            # build and persist index
            storage_context = StorageContext.from_defaults(vector_store=self.vector_store)
            logger.info("indexing the docs in VectorStoreIndex")
            self._index = VectorStoreIndex.from_documents(documents=_docs, storage_context=storage_context,
                                                          show_progress=self.show_progress)

    def do_rag(self, user_query: str) -> RESPONSE_TYPE:

        logger.info("retrieving the relavent nodes")
        query_engine = self._index.as_query_engine(similarity_top_k=self.similarity_top_k)
        logger.info("LLM is thinking...")
        response = query_engine.query(str_or_query_bundle=user_query)
        logger.info(f'response: {response}')
        if os.environ.get('IS_EVALUATION_NEEDED') == 'true':
            self.rag_evaluator.evaluate(user_query=user_query, response_obj=response)
        return response



from llama_deploy import (
    deploy_workflow,
    WorkflowServiceConfig,
    ControlPlaneConfig
)
from retry_guideline_query_engine_workflow import build_rag_workflow_with_retry_guideline_query_engine
from dotenv import load_dotenv, find_dotenv


_ = load_dotenv(find_dotenv())


async def deploy_rag_workflow_with_retry_guideline_query_engine():
    rag_workflow = build_rag_workflow_with_retry_guideline_query_engine()
    try:
        await deploy_workflow(
            workflow=rag_workflow,
            workflow_config=WorkflowServiceConfig(
                host="127.0.0.1",
                port=8004,
                # service name matches the name of the workflow used in Agentic Workflow
                service_name="rag_workflow_with_retry_guideline_query_engine",
                description="RAG workflow",
            ),
            # Config controlled by env vars
            control_plane_config=ControlPlaneConfig()
        )
    except Exception as e:
        print(e)

if __name__ == "__main__":
    import asyncio
    import nest_asyncio

    nest_asyncio.apply()
    try:
        asyncio.run(deploy_rag_workflow_with_retry_guideline_query_engine())
    except Exception as e:
        print(e)



# driver code
from controllable_agents_with_human_in_the_loop import ControllableAgentsWithHumanInLoop


controllable_agent = ControllableAgentsWithHumanInLoop(input_dir='data', show_progress=True)

# Start a loop to continually get input from the user
while True:
    # Get a query from the user
    user_query = input("Enter your query [type 'bye' to 'exit']: ")

    # Check if the user wants to terminate the loop
    if user_query.lower() == "bye" or user_query.lower() == "exit":
        break

    response = controllable_agent.chat_repl(user_query=user_query)
    print(response)


import pytest
from click.testing import CliRunner
from unittest.mock import patch
from bootstraprag.cli import cli, download_and_extract_template  # replace 'your_cli_module' with the actual module name


@pytest.fixture
def runner():
    return CliRunner()


def test_create_project_without_overwrite(runner):
    # Mock Path.exists to simulate that the project directory already exists
    with patch('pathlib.Path.exists', return_value=True), \
            patch('shutil.copytree') as mock_copy, \
            patch('InquirerPy.inquirer.select') as mock_inquirer_select:
        # Ensure copytree wasn't called since the directory exists
        mock_copy.assert_not_called()
        # Ensure no prompt was triggered
        mock_inquirer_select.assert_not_called()

        # The prompts should not be triggered because the function should exit early
        result = runner.invoke(cli, ['create', 'test_project'])

        # Assert that the command exited with success (in this case we expect no project creation)
        assert result.exit_code == 0, f"Exit code was {result.exit_code}, output: {result.output}"
        assert "Error: Project directory test_project already exists!" in result.output


def test_framework_selection_llamaindex(runner):
    with patch('InquirerPy.inquirer.select.execute', return_value='llamaindex'):
        result = runner.invoke(cli, ['create', 'test_project'])
        assert result.exit_code == 0
        assert 'You have selected framework: llamaindex' in result.output


def test_download_and_extract_template_with_observability():
    with patch('shutil.copytree') as mock_copy:
        download_and_extract_template('test_project', 'llamaindex', 'simple-rag', 'Yes')
        mock_copy.assert_called_once()


def test_download_and_extract_template_without_observability():
    with patch('shutil.copytree') as mock_copy:
        download_and_extract_template('test_project', 'llamaindex', 'simple-rag', 'No')
        mock_copy.assert_called_once()



import os

from advanced_hybrid_search import AdvancedHybridSearch
from dotenv import load_dotenv, find_dotenv

_ = load_dotenv(find_dotenv())

adv_search = AdvancedHybridSearch(collection_name=os.environ.get('COLLECTION_NAME'))
# adv_search.insert_data()

query_text = "What is the impact of COVID-19 on the environment?"
results = adv_search.query_with_dense_embedding(query_text=query_text)
print(results)

results = adv_search.query_with_sparse_embedding(query_text=query_text)
print(results)

results = adv_search.query_with_late_interaction_embedding(query_text=query_text)
print(results)

results = adv_search.query_with_rrf(query_text=query_text)
print(results)



import click
import shutil
from pathlib import Path
from InquirerPy import inquirer


@click.group()
def cli():
    pass


# used for downloading the project as zip.
def create_zip(project_name):
    zip_path = shutil.make_archive(project_name, 'zip', project_name)
    return zip_path


@click.command()
@click.argument('project_name')
@click.option('--framework', type=click.Choice([]), prompt=False)
@click.option('--template', type=click.Choice([]), prompt=False)
@click.option('--observability', type=click.Choice([]), prompt=False)
def create(project_name, framework, template, observability):
    template_choices = []
    framework_choices = ['llamaindex', 'None']
    framework = inquirer.select(
        message="Which technology would you like to use ('None' will make you to use qdrant direct search)?",
        choices=framework_choices
    ).execute()
    if framework == 'llamaindex' or framework == 'langchain' or framework == 'haystack':
        template_choices = [
            'simple-rag',
            'rag-with-react',
            'rag-with-hyde',
            'rag-with-flare',
            'rag-with-self-correction',
            'rag-with-controllable-agents',
            'llama-deploy-with-simplemq',
            'llama-deploy-with-rabbitmq',
            'llama-deploy-with-kafka'
        ]
    elif framework == 'None':
        framework = 'qdrant'
        template_choices = ['simple-search', 'multimodal-search', 'hybrid-search', 'hybrid-search-advanced',
                            'retrieval-quality']
    # Use InquirerPy to select template with arrow keys
    template = inquirer.select(
        message="Which template would you like to use?",
        choices=template_choices,
    ).execute()
    if framework == 'llamaindex' or framework == 'langchain' or framework == 'haystack':
        observability_choices = ['Yes', 'No']
        # Use InquirerPy to select observability with arrow keys
        observability = inquirer.select(
            message="Do you wish to enable observability?",
            choices=observability_choices,
        ).execute()

    click.echo(f'You have selected framework: {framework} and template: {template} and observability: {observability}')
    download_and_extract_template(project_name, framework, template, observability)


def download_and_extract_template(project_name, framework, template, observability_selection):
    if observability_selection == 'Yes':
        folder_name = template.replace('-', '_') + '_with_observability'
        base_path = Path(__file__).parent / 'templates' / framework / folder_name
    else:
        base_path = Path(__file__).parent / 'templates' / framework / str(template).replace('-', '_')
    project_path = Path.cwd() / project_name

    if project_path.exists():
        click.echo(f"Error: Project directory {project_name} already exists!")
        return

    try:
        shutil.copytree(base_path, project_path)
        click.echo(f"Project {project_name} created successfully at {project_path}")
    except Exception as e:
        click.echo(f"Error: {e}")


cli.add_command(create)

if __name__ == "__main__":
    cli()



import os
import logging
import qdrant_client
from llama_index.core.workflow import (
    Workflow,
    Context,
    StartEvent,
    StopEvent,
    step
)
from llama_index.core.base.base_query_engine import BaseQueryEngine
from llama_index.core.query_engine import RetryGuidelineQueryEngine
from llama_index.core import (VectorStoreIndex, Settings, StorageContext, SimpleDirectoryReader)
from llama_index.core.evaluation import GuidelineEvaluator
from llama_index.vector_stores.qdrant import QdrantVectorStore
from llama_index.llms.ollama import Ollama
from llama_index.embeddings.ollama import OllamaEmbedding
from llama_index.core.evaluation.guideline import DEFAULT_GUIDELINES
from dotenv import load_dotenv, find_dotenv
from events import QueryEngineEvent

_ = load_dotenv(find_dotenv())

logging.basicConfig(level=int(os.environ['INFO']))
logger = logging.getLogger(__name__)


class RAGWorkflowWithRetryGuidelineQueryEngine(Workflow):
    def __init__(self, index: VectorStoreIndex, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.index: VectorStoreIndex = index

    @step
    async def create_retry_query_engine(self, ctx: Context, ev: StartEvent) -> QueryEngineEvent | None:
        "Entry point for RAG, triggered by a StartEvent with `query`."
        logger.info(f"creating query engine for query: {ev.get('query')}")
        query = ev.get("query")
        no_of_retries = ev.get("no_of_retries", default=3)

        if not query:
            raise ValueError("Query is required!")

        # store the settings in the global context
        await ctx.set("query", query)
        await ctx.set("no_of_retries", no_of_retries)

        base_query_engine = self.index.as_query_engine(llm=Settings.llm, similarity_top_k=2, sparse_top_k=12,
                                                       vector_store_query_mode="hybrid")
        return QueryEngineEvent(base_query_engine=base_query_engine)

    @step
    async def query_with_retry_source_query_engine(self, ctx: Context, ev: QueryEngineEvent) -> StopEvent:
        """Return a response using reranked nodes."""
        query = await ctx.get("query")
        no_of_retries = await ctx.get("no_of_retries")
        base_query_engine: BaseQueryEngine = ev.base_query_engine

        # Guideline eval
        guideline_eval = GuidelineEvaluator(
            guidelines=DEFAULT_GUIDELINES + "\nThe response should not be overly long.\n"
                                            "The response should try to summarize where possible.\n"
        )  # just for example
        retry_guideline_query_engine = RetryGuidelineQueryEngine(base_query_engine, guideline_eval,
                                                                 resynthesize_query=True, max_retries=no_of_retries)
        retry_guideline_response = retry_guideline_query_engine.query(query)
        logger.info(f"response for query is: {retry_guideline_response}")
        return StopEvent(result=str(retry_guideline_response))


def build_rag_workflow_with_retry_guideline_query_engine() -> RAGWorkflowWithRetryGuidelineQueryEngine:
    index_loaded = False
    # host points to qdrant in docker-compose.yml
    client = qdrant_client.QdrantClient(url=os.environ['DB_URL'], api_key=os.environ['DB_API_KEY'])
    aclient = qdrant_client.AsyncQdrantClient(url=os.environ['DB_URL'], api_key=os.environ['DB_API_KEY'])
    vector_store = QdrantVectorStore(collection_name=os.environ['COLLECTION_NAME'], client=client, aclient=aclient,
                                     enable_hybrid=True, batch_size=50)

    Settings.llm = Ollama(model=os.environ['OLLAMA_LLM_MODEL'], base_url=os.environ['OLLAMA_BASE_URL'],
                          request_timeout=600)
    Settings.embed_model = OllamaEmbedding(model_name=os.environ['OLLAMA_EMBED_MODEL'],
                                           base_url=os.environ['OLLAMA_BASE_URL'])

    # index = VectorStoreIndex.from_vector_store(vector_store=vector_store, embed_model=Settings.embed_model)
    index: VectorStoreIndex = None

    if client.collection_exists(collection_name=os.environ['COLLECTION_NAME']):
        try:
            index = VectorStoreIndex.from_vector_store(vector_store=vector_store)
            index_loaded = True
        except Exception as e:
            index_loaded = False

    if not index_loaded:
        # load data
        _docs = (SimpleDirectoryReader(input_dir='data', required_exts=['.pdf']).load_data(show_progress=True))

        # build and persist index
        storage_context = StorageContext.from_defaults(vector_store=vector_store)
        logger.info("indexing the docs in VectorStoreIndex")
        index = VectorStoreIndex.from_documents(documents=_docs, storage_context=storage_context, show_progress=True)

    return RAGWorkflowWithRetryGuidelineQueryEngine(index=index, timeout=120.0)



import pytest
import subprocess
import platform
from utils.tools_utils import get_ip_address


def mock_subprocess_run_ifconfig(*args, **kwargs):
    # Mock output for Linux/macOS `ifconfig` command
    class MockCompletedProcess:
        def __init__(self):
            self.stdout = """
            en0: flags=8863<UP,BROADCAST,SMART,RUNNING,SIMPLEX,MULTICAST> mtu 1500
                inet 192.168.1.2 netmask 0xffffff00 broadcast 192.168.1.255
            lo0: flags=8049<UP,LOOPBACK,RUNNING,MULTICAST> mtu 16384
                inet 127.0.0.1 netmask 0xff000000
            """

    return MockCompletedProcess()


def mock_subprocess_run_ipconfig(*args, **kwargs):
    # Mock output for Windows `ipconfig` command
    class MockCompletedProcess:
        def __init__(self):
            self.stdout = """
            Ethernet adapter Ethernet:
                IPv4 Address. . . . . . . . . . . : 192.168.1.100
                Subnet Mask . . . . . . . . . . . : 255.255.255.0
                Default Gateway . . . . . . . . . : 192.168.1.1
            """

    return MockCompletedProcess()


@pytest.mark.skipif(platform.system() == "Windows", reason="Running on Windows OS")
def test_get_ip_address_linux_or_mac(monkeypatch):
    if platform.system() in ["Linux", "Darwin"]:
        # Mock subprocess.run to return Linux/macOS output
        monkeypatch.setattr(subprocess, 'run', mock_subprocess_run_ifconfig)

        # Run the get_ip_address function and assert the result
        result = get_ip_address()
        assert result == "IPv4 Address: 192.168.1.2"


@pytest.mark.skipif(platform.system() != "Windows", reason="Not running on Windows OS")
def test_get_ip_address_windows(monkeypatch):
    if platform.system() == "Windows":
        # Mock subprocess.run to return Windows output
        monkeypatch.setattr(subprocess, 'run', mock_subprocess_run_ipconfig)

        # Run the get_ip_address function and assert the result
        result = get_ip_address()
        assert result == "IPv4 Address: 192.168.1.100"



from abc import ABC
from dotenv import load_dotenv, find_dotenv
from self_correction_core import SelfCorrectingRAG
import litserve as ls
import os

_ = load_dotenv(find_dotenv())


class SimpleRAGServingAPI(ls.LitAPI, ABC):
    def __init__(self):
        self.self_correcting_rag = None
        self.method: str | None = None

    def setup(self, devices):
        self.method = 'retry_query_engine'  # default query engine
        self.self_correcting_rag = SelfCorrectingRAG(input_dir='data', show_progress=True, no_of_retries=3)

    def decode_request(self, request, **kwargs):
        if "method" in request.keys():
            self.method = request["method"]
        return request["query"]

    def predict(self, query: str):
        try:
            if self.method == 'retry_query_engine':
                return self.self_correcting_rag.query_with_retry_query_engine(query=query)
            elif self.method == 'source_query_engine':
                return self.self_correcting_rag.query_with_source_query_engine(query=query)
            elif self.method == 'guideline_query_engine':
                return self.self_correcting_rag.query_with_guideline_query_engine(query=query)
            else:
                raise Exception('No a proper method passed')
        except Exception as e:
            return e.args[0]

    def encode_response(self, output, **kwargs):
        return {'response': output}


if __name__ == '__main__':
    api = SimpleRAGServingAPI()
    server = ls.LitServer(lit_api=api, api_path='/api/v1/chat-completion',
                          workers_per_device=int(os.environ.get('LIT_SERVER_WORKERS_PER_DEVICE')))
    server.run(port=os.environ.get('LIT_SERVER_PORT'))



from setuptools import setup, find_packages
import pathlib

doc_path = pathlib.Path(__file__).parent.resolve()
long_description = (doc_path / "README.md").read_text(encoding="utf-8")

setup(
    name='bootstrap-rag',
    version='0.0.8',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'click',
    ],
    entry_points={
        'console_scripts': [
            'bootstraprag=bootstraprag.cli:cli',
        ],
    },
    package_data={
        'bootstraprag': ['templates/*'],
    },
)


from llama_deploy import LlamaDeployClient, ControlPlaneConfig

# points to deployed control plane
client = LlamaDeployClient(ControlPlaneConfig())

query = "what are the challenges of mlops?"
session = client.create_session()

result3 = session.run(service_name="rag_workflow_with_retry_guideline_query_engine", query=query)

print(f'response from rag_workflow_with_retry_guideline_query_engine is {result3}')
